@@ -2,10 +2,8 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (C) 2010-2013 celeron55, Perttu Ahola <celeron55@gmail.com>
 
+#include <map>
 #include "irrlichttypes_bloated.h"
-#include "irrlicht.h" // createDevice
-#include "irrlicht_changes/printing.h"
-#include "benchmark/benchmark.h"
 #include "chat_interface.h"
 #include "debug.h"
 #include "unittest/test.h"
@@ -17,6 +15,7 @@
 #include "gettext.h"
 #include "log.h"
 #include "log_internal.h"
+#include "util/serialize.h"
 #include "util/quicktune.h"
 #include "httpfetch.h"
 #include "gameparams.h"
@@ -25,16 +24,20 @@
 #include "player.h"
 #include "porting.h"
 #include "serialization.h" // SER_FMT_VER_HIGHEST_*
+#include "serverenvironment.h"
+#include "servermap.h"
+#include "settings.h"
 #include "network/socket.h"
+#include "network/networkexceptions.h"
 #include "mapblock.h"
 #if USE_CURSES
 	#include "terminal_chat_console.h"
 #endif
 #if CHECK_CLIENT_BUILD()
-#include "gui/guiMainMenu.h"
 #include "client/clientlauncher.h"
-#include "gui/guiEngine.h"
-#include "gui/mainmenumanager.h"
+#endif
+#if BUILD_BENCHMARKS
+#include "benchmark/benchmark.h"
 #endif
 
 // for version information only
@@ -68,6 +71,7 @@ extern "C" {
 #define ENV_NO_COLOR "NO_COLOR"
 #define ENV_CLICOLOR "CLICOLOR"
 #define ENV_CLICOLOR_FORCE "CLICOLOR_FORCE"
+#define ENV_LOG_TIMESTAMP "LOG_TIMESTAMP"
 
 typedef std::map<std::string, ValueSpec> OptionList;
 
@@ -290,7 +294,7 @@ static void get_env_opts(Settings &args)
 	// CLICOLOR != 0: ANSI colors are supported (auto-detection, this is the default)
 	// CLICOLOR == 0: ANSI colors are NOT supported
 	const char *clicolor = std::getenv(ENV_CLICOLOR);
-	if (clicolor && std::string(clicolor) == "0") {
+	if (clicolor && std::string_view(clicolor) == "0") {
 		args.set("color", "never");
 	}
 	// NO_COLOR only specifies that no color is allowed.
@@ -301,10 +305,15 @@ static void get_env_opts(Settings &args)
 	}
 	// CLICOLOR_FORCE is another option, which should turn on colors "no matter what".
 	const char *clicolor_force = std::getenv(ENV_CLICOLOR_FORCE);
-	if (clicolor_force && std::string(clicolor_force) != "0") {
+	if (clicolor_force && std::string_view(clicolor_force) != "0") {
 		// should ALWAYS have colors, so we ignore tty (no "auto")
 		args.set("color", "always");
 	}
+
+	// No standard, Luanti-specific
+	const char *log_ts = std::getenv(ENV_LOG_TIMESTAMP);
+	if (log_ts)
+		args.set("log-timestamp", log_ts);
 }
 
 static bool get_cmdline_opts(int argc, char *argv[], Settings *cmd_args)
@@ -352,8 +361,10 @@ static void set_allowed_options(OptionList *allowed_options)
 			"'name' lists names, 'both' lists both)"))));
 	allowed_options->insert(std::make_pair("quiet", ValueSpec(VALUETYPE_FLAG,
 			_("Print only errors to console"))));
-	allowed_options->insert(std::make_pair("color", ValueSpec(VALUETYPE_STRING,
-			_("Coloured logs ('always', 'never' or 'auto'), defaults to 'auto'"))));
+	allowed_options->emplace("color", ValueSpec(VALUETYPE_STRING,
+			_("Coloured logs ('always', 'never' or 'auto'), default: 'auto'")));
+	allowed_options->emplace("log-timestamp", ValueSpec(VALUETYPE_STRING,
+			_("Timestamped logs ('wall', 'relative' or 'none'), default: 'wall'")));
 	allowed_options->insert(std::make_pair("info", ValueSpec(VALUETYPE_FLAG,
 			_("Print more information to console"))));
 	allowed_options->insert(std::make_pair("verbose",  ValueSpec(VALUETYPE_FLAG,
@@ -405,6 +416,22 @@ static void print_help(const OptionList &allowed_options)
 {
 	std::cout << _("Allowed options:") << std::endl;
 	print_allowed_options(allowed_options);
+	std::cout << std::endl;
+
+	std::pair<const char*, std::vector<std::string>> the_list[] = {
+		{"map", ServerMap::getDatabaseBackends()},
+		{"players", ServerEnvironment::getPlayerDatabaseBackends()},
+		{"auth", ServerEnvironment::getAuthDatabaseBackends()},
+		{"mod storage", Server::getModStorageDatabaseBackends()},
+	};
+
+	std::cout << "Supported database backends:";
+	for (auto &e : the_list) {
+		SORT_AND_UNIQUE(e.second);
+		std::cout << "\n  " << padStringRight(e.first, 16)
+			<< ": " << str_join(e.second, ", ");
+	}
+	std::cout << std::endl;
 }
 
 static void print_allowed_options(const OptionList &allowed_options)
@@ -505,11 +532,9 @@ static bool setup_log_params(const Settings &cmd_args)
 		g_logger.addOutputMaxLevel(&stderr_output, LL_ERROR);
 	}
 
-	// Coloured log messages (see log.h)
+	// Message color
 	std::string color_mode;
-	if (cmd_args.exists("color")) {
-		color_mode = cmd_args.get("color");
-	}
+	cmd_args.getNoEx("color", color_mode);
 	if (!color_mode.empty()) {
 		if (color_mode == "auto") {
 			Logger::color_mode = LOG_COLOR_AUTO;
@@ -523,6 +548,22 @@ static bool setup_log_params(const Settings &cmd_args)
 		}
 	}
 
+	// Timestamp
+	std::string ts_mode;
+	cmd_args.getNoEx("log-timestamp", ts_mode);
+	if (!ts_mode.empty()) {
+		if (ts_mode == "wall") {
+			Logger::timestamp_mode = LOG_TIMESTAMP_WALL;
+		} else if (ts_mode == "relative") {
+			Logger::timestamp_mode = LOG_TIMESTAMP_RELATIVE;
+		} else if (ts_mode == "none") {
+			Logger::timestamp_mode = LOG_TIMESTAMP_NONE;
+		} else {
+			errorstream << "Invalid timestamp mode: " << ts_mode << std::endl;
+			return false;
+		}
+	}
+
 	// In certain cases, output info level on stderr
 	if (cmd_args.getFlag("info") || cmd_args.getFlag("verbose") ||
 			cmd_args.getFlag("trace") || cmd_args.getFlag("speedtests"))
@@ -1000,8 +1041,7 @@ static bool get_game_from_cmdline(GameParams *game_params, const Settings &cmd_a
 			errorstream << "Game \"" << gameid << "\" not found" << std::endl;
 			return false;
 		}
-		dstream << _("Using game specified by --gameid on the command line")
-		        << std::endl;
+		infostream << "Using commanded gameid [" << commanded_gamespec.id << "]" << std::endl;
 		game_params->game_spec = commanded_gamespec;
 		return true;
 	}
@@ -1011,18 +1051,19 @@ static bool get_game_from_cmdline(GameParams *game_params, const Settings &cmd_a
 
 static bool determine_subgame(GameParams *game_params)
 {
-	SubgameSpec gamespec;
+	if (!game_params->is_dedicated_server) {
+		// ClientLauncher has its own logic to choose a game
+		return true;
+	}
 
+	SubgameSpec gamespec;
 	assert(!game_params->world_path.empty());	// Pre-condition
 
-	// If world doesn't exist
-	if (!game_params->world_path.empty()
-		&& !getWorldExists(game_params->world_path)) {
+	if (!getWorldExists(game_params->world_path)) {
 		// Try to take gamespec from command line
 		if (game_params->game_spec.isValid()) {
 			gamespec = game_params->game_spec;
-			infostream << "Using commanded gameid [" << gamespec.id << "]" << std::endl;
-		} else if (game_params->is_dedicated_server) {
+		} else {
 			auto games = getAvailableGameIds();
 			// If there's exactly one obvious choice then do the right thing
 			if (games.size() > 1)
@@ -1053,16 +1094,12 @@ static bool determine_subgame(GameParams *game_params)
 				            << world_gameid << "]" << std::endl;
 			}
 		} else {
-			// If world contains an embedded game, use it;
-			// Otherwise find world from local system.
 			gamespec = findWorldSubgame(game_params->world_path);
 			infostream << "Using world gameid [" << gamespec.id << "]" << std::endl;
 		}
 	}
 
 	if (!gamespec.isValid()) {
-		if (!game_params->is_dedicated_server)
-			return true; // not an error, this would prevent the main menu from running
 		errorstream << "Game [" << gamespec.id << "] could not be found."
 		            << std::endl;
 		return false;
