@@ -7,7 +7,6 @@
 #include "lua_api/l_settings.h"
 #include "common/c_converter.h"
 #include "common/c_content.h"
-#include "cpp_api/s_async.h"
 #include "network/networkprotocol.h"
 #include "serialization.h"
 #include <json/json.h>
@@ -15,7 +14,6 @@
 #include "cpp_api/s_security.h"
 #include "porting.h"
 #include "convert_json.h"
-#include "debug.h"
 #include "log.h"
 #include "log_internal.h"
 #include "tool.h"
@@ -54,7 +52,13 @@ int ModApiUtil::l_log(lua_State *L)
 		auto name = readParam<std::string_view>(L, 1);
 		text = readParam<std::string_view>(L, 2);
 		if (name == "deprecated") {
-			log_deprecated(L, text, 2);
+			// core.log("deprecated", message [, stack_level])
+			// Level 1 - immediate caller of core.log (probably engine code);
+			// Level 2 - caller of the function that called core.log, and so on
+			int stack_level = readParam<int>(L, 3, 2);
+			if (stack_level < 1)
+				throw LuaError("invalid stack level");
+			log_deprecated(L, text, stack_level);
 			return 0;
 		}
 		level = Logger::stringToLevel(name);
@@ -191,12 +195,12 @@ int ModApiUtil::l_get_dig_params(lua_State *L)
 int ModApiUtil::l_get_hit_params(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
-	std::unordered_map<std::string, int> groups;
+	ItemGroupList groups;
 	read_groups(L, 1, groups);
 	ToolCapabilities tp = read_tool_capabilities(L, 2);
 	float time_from_last_punch = readParam<float>(L, 3, 1000000);
 	int wear = readParam<int>(L, 4, 0);
-	push_hit_params(L, getHitParams(groups, &tp,
+	push_hit_params(L, getHitParams(groups, tp,
 		time_from_last_punch, wear));
 	return 1;
 }
@@ -257,6 +261,21 @@ int ModApiUtil::l_is_yes(lua_State *L)
 	return 1;
 }
 
+// path_exists(path)
+int ModApiUtil::l_path_exists(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	std::string path = luaL_checkstring(L, 1); //path
+
+	CHECK_SECURE_PATH(L, path.c_str(), false);
+
+	bool exists = fs::PathExists(path);
+	lua_pushboolean(L, exists);
+
+	return 1;
+}
+
 // get_builtin_path()
 int ModApiUtil::l_get_builtin_path(lua_State *L)
 {
@@ -573,7 +592,7 @@ int ModApiUtil::l_colorspec_to_colorstring(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
 
-	video::SColor color(0);
+	video::SColor color;
 	if (read_color(L, 1, &color)) {
 		char colorstring[10];
 		snprintf(colorstring, 10, "#%02X%02X%02X%02X",
@@ -590,7 +609,7 @@ int ModApiUtil::l_colorspec_to_bytes(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
 
-	video::SColor color(0);
+	video::SColor color;
 	if (read_color(L, 1, &color)) {
 		u8 colorbytes[4] = {
 			(u8) color.getRed(),
@@ -610,7 +629,7 @@ int ModApiUtil::l_colorspec_to_table(lua_State *L)
 {
 	NO_MAP_LOCK_REQUIRED;
 
-	video::SColor color(0);
+	video::SColor color;
 	if (read_color(L, 1, &color)) {
 		push_ARGB8(L, color);
 		return 1;
@@ -689,6 +708,17 @@ int ModApiUtil::l_is_valid_player_name(lua_State *L)
 	return 1;
 }
 
+// strip_escapes(str)
+int ModApiUtil::l_strip_escapes(lua_State *L)
+{
+	NO_MAP_LOCK_REQUIRED;
+
+	auto s = readParam<std::string_view>(L, 1);
+	auto r = unescape_enriched(s);
+	lua_pushlstring(L, r.c_str(), r.size());
+	return 1;
+}
+
 void ModApiUtil::Initialize(lua_State *L, int top)
 {
 	API_FCT(log);
@@ -707,6 +737,8 @@ void ModApiUtil::Initialize(lua_State *L, int top)
 
 	API_FCT(is_yes);
 
+	API_FCT(path_exists);
+
 	API_FCT(get_builtin_path);
 	API_FCT(get_user_path);
 
@@ -740,6 +772,7 @@ void ModApiUtil::Initialize(lua_State *L, int top)
 
 	API_FCT(urlencode);
 	API_FCT(is_valid_player_name);
+	API_FCT(strip_escapes);
 
 	LuaSettings::create(L, g_settings, g_settings_path);
 	lua_setfield(L, top, "settings");
@@ -774,6 +807,7 @@ void ModApiUtil::InitializeClient(lua_State *L, int top)
 	API_FCT(set_last_run_mod);
 
 	API_FCT(urlencode);
+	API_FCT(strip_escapes);
 
 	LuaSettings::create(L, g_settings, g_settings_path);
 	lua_setfield(L, top, "settings");
@@ -790,6 +824,8 @@ void ModApiUtil::InitializeAsync(lua_State *L, int top)
 
 	API_FCT(is_yes);
 
+	API_FCT(path_exists);
+
 	API_FCT(get_builtin_path);
 	API_FCT(get_user_path);
 
@@ -822,6 +858,7 @@ void ModApiUtil::InitializeAsync(lua_State *L, int top)
 	API_FCT(set_last_run_mod);
 
 	API_FCT(urlencode);
+	API_FCT(strip_escapes);
 
 	LuaSettings::create(L, g_settings, g_settings_path);
 	lua_setfield(L, top, "settings");
